;*********************************************************************************************
;	Tetris on the Vanguard hardware from Rock-Ola (Vanguard game was 
;	made 1981 by the SNK Corporation)

;	Great fun to program such an old arcade machine !!

;	This file contains many subroutines

;	Norbert Kehrer, October 1998
;*********************************************************************************************


;**************************************************
;	**** DeleteFullRows
;			Delete the rows filled by the landed block
;**************************************************

DeleteFullRows
		lda #$00		; it suffices to check 4 rows starting from y-coo
		sta dfr_cnt
		lda block_y
		sta current_row

dfr_loop	jsr CheckRow		; is the row full?
		cmp #TRUE
		bne dfr_next
		jsr DeleteRow		; yes, away with it

		lda dfr_cnt
		asl a			; score increase is count*2
		jsr IncScore		; increase score

		inc filled_rows		; increment the counter for the filled rows

		lda filled_rows
		cmp #SPEED_INCREASE	; do we need to increase the speed?
		bne dfr_next		; no, go on

		dec game_speed		; yes, increase speed
		cmp #MAXIMUM_SPEED	; but only to the maximum
		bne dfr_speedok
		inc game_speed

dfr_speedok
		lda #0
		sta filled_rows		; reset filled rows counter

dfr_next	inc current_row		; current_row++
		lda current_row
		cmp #31			; stop at row 31 of the playfield
		beq dfr_leave
		bpl dfr_leave
		inc dfr_cnt		; dfr_cnt++
		lda dfr_cnt
		cmp #4			; only 4 rows have to be ckecked
		bne dfr_loop

dfr_leave	
		rts



;**************************************************
;	**** CheckRow	in: current_row
;			out: a (0=delete, 1=do not delete)
;			Check if the row is full, so that it can be deleted
;**************************************************

CheckRow
		lda #1
		sta current_col
		
cr_loop		ldx current_col		; loop over positions of current row
		ldy current_row
		jsr GetChar		; get character there
		cmp #$20		; is it space ?
		beq cr_leave		; yes, then leave with FALSE
		cmp #$00		; is it space 	?
		beq cr_leave		; yes, then leave with FALSE
		cmp #$80		; is it space from a formerly moving object?
		beq cr_leave		; yes, then leave with FALSE

cr_goon		inc current_col		; else, increment column
		lda current_col		
		cmp #PLAYF_WIDTH+1	; until playfield width
		bne cr_loop

		lda #TRUE		; all positions non-space - row deletable!
		rts

cr_leave	lda #FALSE		; one character is space
		rts			; leave with false


;**************************************************
;	**** DeleteRow	in: current_row
;			Delete the row with a beautiful effect
;**************************************************

DeleteRow

		lda #SND_FULLROW	; start sound for full row
		ldx #LEN_FULLROW	; length of sound
		jsr MakeSound

		lda #$60		; *** First, the effect
		pha			; counter is on stack
	
dr_loop_eff	lda #1
		sta current_col
		
dr_loop_eff1	ldx current_col		; loop over positions of current row
		ldy current_row
		jsr GetChar		; get char - h_word_A has address of character now
					;            color RAM address is in h_word_B

		pla			; get current counter
		pha
		and #$01		; make it 0 or 1
		ldy #0
		sta (h_word_A),y	; and store it at the position

		inc current_col		; increment column
		lda current_col		
		cmp #PLAYF_WIDTH+1	; until playfield width
		bne dr_loop_eff1

		pla
		tax
		dex
		txa			; decrement counter
		pha
		cmp #0			; until counter is zero
		bne dr_loop_eff

		pla			; clear stack


		lda #1			; *** second, correction of the blocks above
		sta current_col
		
dr_loop_corr1	ldx current_col		; loop over positions of current row
		ldy current_row
		dey			; line above the current one
		jsr GetChar		; get char - h_word_A has address of character now
					;            color RAM address is in h_word_B


		and #$0f		; only characters 0-16 are used
		tax
		lda after_above,x	; get corrected character

		ldy #0
		sta (h_word_A),y	; and store it at the position

		inc current_col		; increment column
		lda current_col		
		cmp #PLAYF_WIDTH+1	; until playfield width
		bne dr_loop_corr1


		lda current_row		; *** third, correction of the blocks below
		cmp #30			; but only if not the last row
		beq dr_del_row

		lda #1			; ok, correction of the blocks below
		sta current_col
		
dr_loop_corr2	ldx current_col		; loop over positions of current row
		ldy current_row
		iny			; line below the current one
		jsr GetChar		; get char - h_word_A has address of character now
					;            color RAM address is in h_word_B

		and #$0f		; only characters 0-16 are used
		tax
		lda after_below,x	; get corrected character

		ldy #0
		sta (h_word_A),y	; and store it at the position

		inc current_col		; increment column
		lda current_col		
		cmp #PLAYF_WIDTH+1	; until playfield width
		bne dr_loop_corr2



dr_del_row
		lda #1			; *** now, delete the row
		sta current_col
		
dr_loop_a	ldx current_col		; loop over positions of current row
		ldy #0
		jsr GetChar		; get char - h_word_A has address of top character now
					;            color RAM address is in h_word_B
	
		ldy current_row
dr_loop_b	dey			; loop to move all characters above the row
		lda (h_word_A),y	; down by one row
		iny			
		sta (h_word_A),y	; char=char above

		dey			; color RAM also
		lda (h_word_B),y
		iny			
		sta (h_word_B),y

		dey			; y=y-1
		cpy #$00		; until y <= 0
		bne dr_loop_b

		ldx current_col
		ldy #0
		lda #$20
		jsr PutChar		; put a space on the top of the column

		inc current_col		; increment column
		lda current_col		
		cmp #PLAYF_WIDTH+1	; until playfield width
		bne dr_loop_a

		rts


;**************************************************
;	**** GenerateBlock
;			Generate random new block
;**************************************************

GenerateBlock
		lda #DRAW		; set to draw mode
		sta block_draw_mode
		lda #MOVING		; set to moving mode
		sta block_move_mode
		lda #VRAM_2		; set to video RAM 2
		sta vram_selector
		lda #FALSE		; initialize movement flags
		sta push_left
		sta push_right
		sta push_turn
		sta push_down

		lda #5			; set coordinates
		sta block_x
		lda #0
		sta block_y
		lda #0
		sta block_dir

		lda #FALSE
		sta do_move_down	;  do not move down now

		lda rand_var		; generate random number - Thanks to Chris Busch
		sta rand_b
		asl rand_b
		asl rand_b
		clc
		adc rand_b
		asl rand_b
		asl rand_b
		clc
		adc rand_b
		tax
		inx
		stx rand_var		; x = random number now

		lda block_type_arr,x	; get random block in array of random numbers for block types
		sta block_type		; and this is our next block

		jsr ClearPreview	; clear preview area

		lda #PLAYF_WIDTH+3	; set coordinates for preview
		sta block_x
		lda #0
		sta block_y
		lda #0
		sta block_dir

		jsr DrawBlock		; draw the block in the preview area

		lda block_type		; swap block_type and block_type_nxt
		pha
		lda block_type_nxt
		sta block_type
		pla
		sta block_type_nxt
		
		lda #BLOCK_START_X	; set coordinates for playfield
		sta block_x
		lda #0
		sta block_y
		lda #0
		sta block_dir

		jsr DrawBlock		; draw the block in the playfield



		rts



;**************************************************
;	**** DrawPlayfield
;			Draw the playfield
;**************************************************

DrawPlayfield
		jsr ClearPreview	; clear the preview area

		lda #5
		jsr SetColor

		ldx #0
		ldy #0
		lda #(str_field0 & $ff)
		sta print_addr
		lda #(str_field0 >> 8)
		sta print_addr+1
		jsr PrintString		; print an empty line at y=0

		ldx #0
		ldy #1
		lda #(str_field0 & $ff)
		sta print_addr
		lda #(str_field0 >> 8)
		sta print_addr+1
		jsr PrintString		; print an empty line at y=1

		ldx #0
		ldy #2
		lda #(str_field3 & $ff)
		sta print_addr
		lda #(str_field3 >> 8)
		sta print_addr+1
		jsr PrintString		; print top of wall line at y=2

		lda #2			; start with y=2
		pha
dpf_loop	pla			; current line is on stack
		tay			; move it to y
		iny			; y=y+1
		cpy #31			; draw lines until the bottom of the screen
		beq dpf_loop_end
		tya
		pha
		ldx #$00
		lda #(str_field1 & $ff)
		sta print_addr
		lda #(str_field1 >> 8)
		sta print_addr+1
		jsr PrintString		; print walls and empty space between them
		jmp dpf_loop
dpf_loop_end

		ldx #0
		ldy #31
		lda #(str_field2 & $ff)
		sta print_addr
		lda #(str_field2 >> 8)
		sta print_addr+1
		jsr PrintString		; print the bottom line
		
		rts


;**************************************************
;	**** ClearPreview	
;			Clear the screen area, where the next block is previewed
;**************************************************

ClearPreview	
		ldx #PLAYF_WIDTH+3	; clear preview area
		ldy #0
		lda #$20
		jsr PutChar
		ldx #PLAYF_WIDTH+4	; clear preview area
		ldy #0
		lda #$20
		jsr PutChar
		ldx #PLAYF_WIDTH+5	; clear preview area
		ldy #0
		lda #$20
		jsr PutChar
		ldx #PLAYF_WIDTH+6	; clear preview area
		ldy #0
		lda #$20
		jsr PutChar

		ldx #PLAYF_WIDTH+3	; clear preview area
		ldy #1
		lda #$20
		jsr PutChar
		ldx #PLAYF_WIDTH+4	; clear preview area
		ldy #1
		lda #$20
		jsr PutChar
		ldx #PLAYF_WIDTH+5	; clear preview area
		ldy #1
		lda #$20
		jsr PutChar
		ldx #PLAYF_WIDTH+6	; clear preview area
		ldy #1
		lda #$20
		jsr PutChar

		ldx #PLAYF_WIDTH+3	; clear preview area
		ldy #2
		lda #$20
		jsr PutChar
		ldx #PLAYF_WIDTH+4	; clear preview area
		ldy #2
		lda #$20
		jsr PutChar
		ldx #PLAYF_WIDTH+5	; clear preview area
		ldy #2
		lda #$20
		jsr PutChar
		ldx #PLAYF_WIDTH+6	; clear preview area
		ldy #2
		lda #$20
		jsr PutChar

		ldx #PLAYF_WIDTH+3	; clear preview area
		ldy #3
		lda #$20
		jsr PutChar
		ldx #PLAYF_WIDTH+4	; clear preview area
		ldy #3
		lda #$20
		jsr PutChar
		ldx #PLAYF_WIDTH+5	; clear preview area
		ldy #3
		lda #$20
		jsr PutChar
		ldx #PLAYF_WIDTH+6	; clear preview area
		ldy #3
		lda #$20
		jsr PutChar

		rts


;**************************************************
;	**** DownMover	
;			Move block 1 row down depending on game_speed and push_down
;**************************************************

DownMover
		dec game_speed_cnt	; decrease counter
		lda game_speed_cnt	; check counter
		cmp #$00
		bne dm_leave		; if not yet 0, leave

		lda game_speed		; reset counter to game speed
		ldx push_down		; check if joystick is down
		cpx #TRUE
		bne dm_norm_speed	; no, go on
		
		lda #FAST_SPEED		; yes, use fast speed

dm_norm_speed	sta game_speed_cnt

		lda #TRUE		; we move down now
		sta do_move_down	; therefore set down movement flag

dm_leave
		rts



;**************************************************
;	**** MoveBlock	in: a=movement type
;			out: a=movement possible (0=possible, 1=impossible)
;			move block if possible
;**************************************************

MoveBlock
		cmp #MOVE_NOT
		bne mb_goon
		lda #TRUE		; return "movement possible"
		rts			; leave immediately, if no movement

mb_goon
		sta h_this_move		; save movement

		ldx block_x
		stx block_x_old		; save x-coo
		ldx block_y
		stx block_y_old		; save y-coo
		ldx block_dir
		stx block_dir_old	; save direction

		cmp #MOVE_LEFT		; left
		bne mb_01
		dec block_x

mb_01		cmp #MOVE_RIGHT		; right
		bne mb_02
		inc block_x

mb_02		cmp #MOVE_DOWN		; down
		bne mb_03
		inc block_y

mb_03		cmp #MOVE_TURN		; turn
		bne mb_04
		ldx block_dir		; increment direction, restart with 0 after 3
		inx
		txa
		and #$03
		sta block_dir

mb_04
		jsr CheckBlock		; check if new block position is possible
		cmp #TRUE
		beq mb_moveit		; yes, move

		lda h_this_move		; no, check movement
		cmp #MOVE_TURN		; was it a turn?
		bne mb_restore		; no, restore everything

		inc block_x		; yes, try with x++

		jsr CheckBlock		; check if new block position is possible
		cmp #TRUE
		beq mb_moveit		; yes, move

		dec block_x		; no, try with x--
		dec block_x

		jsr CheckBlock		; check if new block position is possible
		cmp #TRUE
		beq mb_moveit		; yes, move
		
mb_restore
		lda block_x_old		; no, so restore old coordinates
		sta block_x		 
		lda block_y_old
		sta block_y		  
		lda block_dir_old
		sta block_dir

		lda #FALSE		; return "movement impossible"

		rts			; and return
		

mb_moveit
		lda block_x
		sta block_x_new		; save new x-coo
		lda block_y
		sta block_y_new		; save new y-coo
		lda block_dir
		sta block_dir_new	; save new direction

		lda block_x_old		; restore old coordinates
		sta block_x		; for block deletion 
		lda block_y_old
		sta block_y		  
		lda block_dir_old
		sta block_dir

		lda #DELETE		; set to deletion mode
		sta block_draw_mode
		jsr DrawBlock		; delete old block

		lda block_x_new		; new coordinates are the new ones and the
		sta block_x		; old ones
		sta block_x_old
		lda block_y_new
		sta block_y		  
		sta block_y_old		  
		lda block_dir_new
		sta block_dir
		sta block_dir_old		  

		lda #DRAW		; set to draw mode
		sta block_draw_mode
		jsr DrawBlock		; draw new block

		lda #TRUE		; return "movement possible"

		rts



;**************************************************
;	**** GetMovement	out: a=movement type
;				Read input devices and determine movement
;**************************************************

GetMovement	
		lda IN_PORT_0		; read input device
		sta h_in_port_0		; and save value

		and #$80		; joystick left
		beq gm_01
		lda push_left		; is joystick already left?
		cmp #TRUE
		beq gm_011
		lda #TRUE		; no, so set movement flag
		sta push_left
		lda #MOVE_LEFT		; move left and leave
		jmp gm_leave

gm_01		lda #FALSE		; clear flag
		sta push_left
gm_011		lda h_in_port_0		; joystick right
		and #$40
		beq gm_02
		lda push_right		; is joystick already right?
		cmp #TRUE
		beq gm_021
		lda #TRUE		; no, so set movement flag
		sta push_right
		lda #MOVE_RIGHT		; move right and leave
		jmp gm_leave

gm_02		lda #FALSE		; clear flag
		sta push_right
gm_021		lda h_in_port_0		; push button
		and #$08
		beq gm_03
		lda push_turn		; is button already pushed?
		cmp #TRUE
		beq gm_031
		lda #TRUE		; no, so set movement flag
		sta push_turn
		lda #MOVE_TURN		; turn and leave
		jmp gm_leave

gm_03		lda #FALSE		; clear flag
		sta push_turn
gm_031		lda h_in_port_0		; joystick down
		and #$10
		beq gm_04
		lda #TRUE
		sta push_down		; set down flag (will be checked by down-mover)
		jmp gm_nothing

gm_04		lda #FALSE
		sta push_down		; clear down flag (will be checked by down-mover)

gm_nothing	lda #MOVE_NOT		; nothing was pressed

gm_leave	
		rts



;**************************************************
;	**** DrawBlock	in: block_x, block_y, block_type, block_dir, block_draw_mode, block_move_mode
;			Draw current block in video RAM 
;**************************************************

DrawBlock
		jsr GetBlockDesc

		lda block_type		; set block color according to type of block
		tax
		lda block_color,x
		sta curr_color

		ldy #$00		; draw character 1 of block
		jsr DrawBlockChar
		ldy #$03		; draw character 2 of block
		jsr DrawBlockChar
		ldy #$06		; draw character 3 of block
		jsr DrawBlockChar
		ldy #$09		; draw character 4 of block
		jsr DrawBlockChar
	
		rts



;**************************************************
;	**** DrawBlockChar	in: block_x, block_y, block_desc, y (start of character)
;				Draw one character of the block in video RAM  
;**************************************************

DrawBlockChar
		clc
		lda (block_desc),y	; get x-coo
		adc block_x
		pha

		iny
		clc
		lda (block_desc),y	; get y-coo
		adc block_y
		pha

		iny
		lda (block_desc),y	; get character
		sta h_byte_A

		pla			; fill the parameters for PutChar (x,y,a)
		tay
		pla
		tax
		lda h_byte_A
		and block_draw_mode	; to clear block if block_draw_mode is DELETE
		ora block_move_mode	; to put char+128 if block is currently moving
		jsr PutChar		; draw character

		rts


;**************************************************
;	**** CheckBlock	in: block_x, block_y, block_type, block_dir
;			out: a (0=possible, 1=impossible)
;			Check if block is possible on this place
;**************************************************

CheckBlock
		jsr GetBlockDesc	; get address of block description

		ldy #$00		; Check character 1 of block
		jsr CheckBlockChar
		cmp #TRUE
		bne cb_impossible	; if character impossible - exit 
		ldy #$03		; Check character 2 of block
		jsr CheckBlockChar
		cmp #TRUE
		bne cb_impossible	; if character impossible - exit 
		ldy #$06		; Check character 3 of block
		jsr CheckBlockChar
		cmp #TRUE
		bne cb_impossible	; if character impossible - exit 
		ldy #$09		; Check character 4 of block
		jsr CheckBlockChar
		cmp #TRUE
		bne cb_impossible	; if character impossible - exit 	

		rts			; else return with possible (a=0)

cb_impossible	lda #FALSE		; impossible so a=1 and return

		rts


;**************************************************
;	**** CheckBlockChar	in: block_x, block_y, block_desc, y (start of character)
;				out: a (0=possible, 1=impossible)
;				Check for one character of the block if it is possible 
;				on this place
;**************************************************

CheckBlockChar
		clc
		lda (block_desc),y	; get x-coo
		adc block_x
		pha

		iny
		clc
		lda (block_desc),y	; get y-coo
		adc block_y

		tay			; fill the parameters for GetChar (x,y)
		pla
		tax
		jsr GetChar		; get character (a=character)
		
		cmp #$20
		beq cbc_is_empty	; check if empty character 

		cmp #$00
		beq cbc_is_empty	; check if empty character 

		jmp cbc_not_empty	; else not empty

cbc_is_empty
		lda #TRUE		; yes, so return with possible
		rts			

cbc_not_empty	and #MOVING		; no empty character, so check if it is a character
		beq cbc_not_moving	; of the block currently moving 

		lda #TRUE		; moving character, so return with possible
		rts			

cbc_not_moving	lda #FALSE		; another character, so return with impossible
		rts




;**************************************************
;	**** GetBlockDesc	in: block_type, block_dir
;				Determine block description address and store in block_desc
;**************************************************

GetBlockDesc

		lda #(block_0 & $ff)	;determine starting address of block description
		sta block_desc		;store address of block 0
		lda #(block_0 >> 8)
		sta block_desc+1	

		ldx block_type		;add block description size <block_type> times
gbd_bl_add	cpx #$00		;to get the beginning of the whole block descr.
		beq gbd_bl_add_ok
		clc
		lda block_desc
		adc #48
		sta block_desc
		lda block_desc+1
		adc #0
		sta block_desc+1
		dex
		jmp gbd_bl_add
gbd_bl_add_ok	

		ldx block_dir		;add image description size <block_dir> times
gbd_bl_img	cpx #$00		;to get the beginning of the image description
		beq gbd_bl_img_ok
		clc
		lda block_desc
		adc #12
		sta block_desc
		lda block_desc+1
		adc #0
		sta block_desc+1
		dex
		jmp gbd_bl_img
gbd_bl_img_ok

		rts




;**************************************************
;	**** SetColor	in: a (color)
;			Set current print color (0 to 7);				
;**************************************************

SetColor
		sta curr_color
		rts



;**************************************************
;	**** ClearScreen	no parameters
;				Clear video RAM and color RAM
;**************************************************
				
ClearScreen
		lda vram_selector
		cmp #VRAM_2
		beq cs_vram2

		ldx #$00		; Video RAM 1:
cs_loop1	lda #$00 		; Blank character to vram 1
		sta VIDEO_RAM_1+$0000,x
		sta VIDEO_RAM_1+$0100,x
		sta VIDEO_RAM_1+$0200,x
		sta VIDEO_RAM_1+$0300,x	

		lda COLOR_RAM+$0000,x	; Initialize color bits 0-2
		and #%11111000
		sta COLOR_RAM+$0000,x
		lda COLOR_RAM+$0100,x	
		and #%11111000
		sta COLOR_RAM+$0100,x
		lda COLOR_RAM+$0200,x
		and #%11111000
		sta COLOR_RAM+$0200,x
		lda COLOR_RAM+$0300,x
		and #%11111000
		sta COLOR_RAM+$0300,x
		dex
		bne cs_loop1
cs_vram2
		ldx #$00		; Video RAM 2
cs_loop2	lda #$20 		; Blank character to vram 2
		sta VIDEO_RAM_2+$0000,x
		sta VIDEO_RAM_2+$0100,x
		sta VIDEO_RAM_2+$0200,x
		sta VIDEO_RAM_2+$0300,x	
		lda COLOR_RAM+$0000,x	; Initialize color bits 3-5
		and #%11000111
		sta COLOR_RAM+$0000,x
		lda COLOR_RAM+$0100,x	
		and #%11000111
		sta COLOR_RAM+$0100,x
		lda COLOR_RAM+$0200,x
		and #%11000111
		sta COLOR_RAM+$0200,x
		lda COLOR_RAM+$0300,x
		and #%11000111
		sta COLOR_RAM+$0300,x
		dex
		bne cs_loop2

		rts


;**************************************************
;	**** PrintString	in: x (X-coo), y (Y-coo), print_addr
;				Print string  into video RAM at coordinate using current color
;**************************************************;				

PrintString 
		stx ps_x
		sty ps_y

		ldy #$00
		sty ps_cnt

ps_loop		ldy ps_cnt
		lda (print_addr),y
		cmp #$00		; String ends with null
		beq ps_loop_end	
		ldx ps_x
		ldy ps_y
		jsr PutChar
		inc ps_x
		inc ps_cnt
		bne ps_loop
ps_loop_end
		rts		
		








;**************************************************
;	**** PutChar	in: x (X-coo), y (Y-coo), a (character)
;			Put character into video RAM at coordinate using current color
;**************************************************

PutChar
		pha
		stx h_byte_A
		clc
		lda #28		;calculate position: p = (28-x)*32 + y
		sbc h_byte_A
		sta h_word_A	;28-x
		lda #$00
		sta h_word_A+1
		clc
		asl h_word_A	;times 2
		rol h_word_A+1
		asl h_word_A	;times 4
		rol h_word_A+1
		asl h_word_A	;times 8
		rol h_word_A+1
		asl h_word_A	;times 16
		rol h_word_A+1
		asl h_word_A	;times 32
		rol h_word_A+1
		clc
		tya		;plus y
		adc h_word_A
		sta h_word_A

		lda #$04		;get video ram start (0400 or 0800)
		sta h_byte_B		;and delta to color ram (0c00)
		lda #$08		;VRAM 2 = 0800	
		ldx vram_selector	;depending on video ram selector
		cpx #VRAM_2
		beq pc_01
		lda #$08		;delta from 0400 to color ram is 08
		sta h_byte_B		
		lda #$04		;video ram 1 = 0400
pc_01	
		clc
		adc h_word_A+1		;calculate position += video ram start
		sta h_word_A+1	

		ldx #$00
		pla
		sta (h_word_A,x)	;store character

		clc
		lda h_byte_B		;plus delta to color ram (0c00)
		adc h_word_A+1
		sta h_word_A+1

		ldx #$00
		lda (h_word_A,x)	;load color ram ... 
		and #%11111000		;clear colors of ram 1 ...
		sta h_byte_A		;and save this
		lda curr_color		;load current color ...
		sta h_byte_B		;and save it

		ldx vram_selector
		cpx #VRAM_1
		beq pc_02		
		asl a			;video RAM 2 uses bits 3 to 5 - so shift
		asl a
		asl a
		sta h_byte_B		; and save
		ldx #$00
		lda (h_word_A,x)	;load color ram ...
		and #%11000111		;clear colors of ram 2 ...
		sta h_byte_A		;and save this
pc_02
		lda h_byte_A		; color = existing color ram | new color ram
		ora h_byte_B		
		ldx #$00
		sta (h_word_A,x)	;store color
		rts		



;**************************************************
;	**** GetChar	in: x (X-coo), y (Y-coo)
;			out: a (character)
;			Get character from video RAM at coordinate
;**************************************************

GetChar
		stx h_byte_A
		clc
		lda #28		;calculate position: p = (28-x)*32 + y
		sbc h_byte_A
		sta h_word_A	;28-x
		lda #$00
		sta h_word_A+1
		clc
		asl h_word_A	;times 2
		rol h_word_A+1
		asl h_word_A	;times 4
		rol h_word_A+1
		asl h_word_A	;times 8
		rol h_word_A+1
		asl h_word_A	;times 16
		rol h_word_A+1
		asl h_word_A	;times 32
		rol h_word_A+1
		clc
		tya		;plus y
		adc h_word_A
		sta h_word_A

		lda #$04		;get video ram start (0400 or 0800)
		sta h_byte_B		;and delta to color ram (0c00)
		lda #$08		;VRAM 2 = 0800	
		ldx vram_selector	;depending on video ram selector
		cpx #VRAM_2
		beq gc_01
		lda #$08		;delta from 0400 to color ram is 08
		sta h_byte_B		
		lda #$04		;video ram 1 = 0400
gc_01	
		clc
		adc h_word_A+1		;calculate position += video ram start
		sta h_word_A+1	

		clc
		lda h_byte_B
		adc h_word_A+1		;color RAM position: video ram start + 0400/0800
		sta h_word_B+1	
		lda h_word_A
		sta h_word_B	

		ldx #$00
		lda (h_word_A,x)	;fetch character

		rts		


;**************************************************
;	**** SaveRegs	
;				God save the registers
;**************************************************

SaveRegs
		php
		pha
		txa
		pha
		tya
		pha
		rts


;**************************************************
;	**** FetchRegs	
;				Fetch the registers back from god's safety
;**************************************************
;				

FetchRegs
		pla
		tay
		pla
		tax
		pla
		plp
		rts




